

                    A PROGRAM FOR FILTERING


     The program listed here reads a record containing a single channel 
of digitized signal, and simultaneously generates a second record of data 
for the filtered signal.  The user may set the number of ordinates per 
cycle at which the high- and low-pass corners exist.  The input record 
must be in straight ASCII form with one floating-point variable per line; 
the file-end marker is several CHR$(4) characters. 

     The program is written in ZBASIC.  The significant digits in 
single, double, and scientific precision are set to maximize speed while 
preserving acceptable accuracy.  The limitation with this program (65,535 
samples) exists because the counter is an unsigned integer variable.  This 
limitation is easily removed. 

     Lines 100 - 990 (not all of them used) are reserved for comments, 
the actual program starting at line 1000.  The graphics screen loaded into 
the computer to introduce the program in line 1010 must be present on the 
program disk, although the program will continue if it is not.  This line 
can be omitted if the introductory screen is not wanted. 

     The first section of the program obtains information from the user.  
It asks for the locations of the input and output data files (the default 
is the "B" drive) and for the names of the input and output files.  
Finally the user must state whether he wishes high-, low-, or bandpass 
filtering, and to supply the periods of the corners.  After calculation of 
the necessary constants (line 1490) the program opens the input and output 
records.  Error trapping is suspended in line 1540 because the program 
supplies its own error traps for end-of-file and filled disk. 

     The header lines are transferred from the input to the output file 
in lines 1550 - 1580 and also printed on the screen.  In the main loop, 
the keyboard status is first checked (line 1700) to see whether the user 
wishes to interrupt the operation (the computer is not responsive to 
<CTRL><BREAK> while the program is running).  

     In bandpass filtering, X1(1) is the current value of the high-pass 
filtered data while X2(1) is that of the low-pass data.  X1(2) and X2(2) 
are obtained by multiplying the next value read from the input record by 
the appropriate constants, and adding a portion of the current values.  
The filtered output, a factor times the difference of X2(2) and X1(2), is 
written immediately to the output record.  During processing, the data 
count is continually shown on the screen (line 1760).  When the end of the 
input record is reached, as signaled by the CHR$(4) characters, a similar 
set of CHR$(4) characters is written to the output record and both records 
are closed.  

     The filtered record usually shows a starting transient which can 
easily be seen in Figure 2, and may disturb operations like finding the 
maximum value in the entire output record.  It may be eliminated by 
applying a Hanning time window to the input data, which can easily be done 
while the work is in progress. 

     The ease with which signals can be filtered suggests that the 
operation can also be done in real time.  Subject to computer and 
program speed limitations, it can be done quite rapidly.  Real-time low-
pass filtering was first used by the writer in June 1986 to eliminate 
noise from the output of a photocell measuring low-level U-V.  The maximum 
frequency in the incoming signal must, of course, be considerably lower 
than the minimum frequency of the interference for low-pass filtering to 
have much beneficial effect. 




00100 '                     SIGNAL FILTERING PROGRAM
00110 '
00120 '        THIS PROGRAM FILTERS A RECORD OF INDEFINITE LENGTH
00130 '             IN SINGLE-PRECISION FLOATING-POINT FORM
00140 '               THE LOW- AND HIGH-FREQUENCY CORNERS
00150 '                   MAY BE SELECTED BY THE USER.
00160 '
00170 '   THIS PROGRAM IS WRITTEN IN ZBASIC AND USES PORTIONS OF CODE
00180 '                    COPYRIGHTED BY ZEDCOR INC.
00190 '            THIS PROGRAM WRITTEN AND COPYRIGHTED 1990
00200 '                       BY CHARLES P. BOEGLI
00210 '
00220 '************************************************************************
00230 '
00240 '                ** THIS VERSION DATED 5/16/90 **
00250 '
00260 '************************************************************************
00270 '
00280 '                           COMPILATION
00290 '
00300 ' SET COMPILER TO THE FOLLOWING DEFAULTS:
00310 ' DOUBLE PRECISION = 8
00320 ' SINGLE PRECISION = 6
00330 ' SCIENTIFIC PRECISION = 6
00340 ' DEFAULT TYPE SINGLE PRECISION
00350 '
00360 '************************************************************************
00370 '
00380 '                          FILE STRUCTURE
00390 '
00400 ' FILES MUST BE IN ASCII FORMAT
00410 ' THE FILE HAS A HEADER OF TWO 20-CHARACTER STRINGS
00420 ' THE FIRST IS THE IDENTIFICATION
00430 ' THE SECOND IS THE UNITS OF MEASUREMENT
00440 ' THEREAFTER THE DATA ARE IN ASCII FORM, ONE VALUE TO A LINE
00450 ' THE FILE-END MARKER IS SEVERAL CHR$(4) CHARACTERS
00460 ' 
00470 '************************************************************************
00480 '
00490 '                     DEFINITION OF VARIABLES
00500 '
00510 ' X = INPUT DATA SAMPLE
00520 ' Y = OUTPUT DATA SAMPLE
00530 ' X1() = INPUT DATA SAMPLE GOING THROUGH HI-PASS FILTER
00540 ' X2() = INPUT DATA SAMPLE GOING THROUGH LO-PASS FILTER
00550 ' F1 = HI-PASS FILTER CONSTANT
00560 ' F2 = LO-PASS FILTER CONSTANT
00570 '
00580 '************************************************************************
00590 '
01000 DEFSTR A,B:DEFINT J,K:DIM X1(2),X2(2),20A(2),12BF(2)
01010 CLS:MODE 7:BLOAD "FILTER.SCR":T=TIMER
01020 IF TIMER<T+5 THEN 1020 ELSE MODE 2
01030 '
01040 '            BUILD NAMES OF INPUT AND OUTPUT FILES
01050 '
01060 CLS:PRINT@(21,4)"DRIVE FOR INPUT DATA FILE (DEFAULT B):
01070 INPUT@(61,4)!&1,"";ANS:ANS=UCASE$(ANS)
01080 LONG IF ANS<>""
01090 IF ASC(ANS)>70 THEN 1070
01100 BDRI=ANS
01110 XELSE
01120 BDRI="B"
01130 END IF
01140 BDRI=BDRI+":":LOCATE 61,4:CLS LINE:PRINT BDRI
01150 PRINT@(21,6)"DRIVE FOR OUTPUT DATA FILE (DEFAULT B):
01160 INPUT@(61,6)!&1,"";ANS:ANS=UCASE$(ANS)
01170 LONG IF ANS<>""
01180 IF ASC(ANS)>70 THEN 1160
01190 BDRO=ANS
01200 XELSE
01210 BDRO="B"
01220 END IF
01230 BDRO=BDRO+":":LOCATE 61,6:CLS LINE:PRINT BDRO
01240 PRINT@(10,8)"NAME OF INPUT DATA FILE, INCLUDING EXTENSION:"
01250 INPUT@(57,8);&12,"";BF(1)
01260 IF INSTR(1,BF(1),".")=0 THEN BF(1)=BF(1)+"."
01270 BF(1)=BDRI+BF(1):LOCATE 57,8:CLS LINE:COLOR,15:PRINT BF(1)
01280 COLOR,7:PRINT@(10,10)"NAME OF OUTPUT DATA FILE:"
01290 INPUT@(36,10);&12,"";BF(2)
01300 IF INSTR(1,BF(2),".")=0 THEN BF(2)=BF(2)+".FLT"
01310 BF(2)=BDRO+BF(2):LOCATE 36,10:CLS LINE:COLOR,15:PRINT BF(2)
01320 '
01330 '                 DETERMINE TYPE OF FILTERING
01340 '
01350 COLOR,7:PRINT@(5,14)"SELECT (1) BANDPASS, (2) LOWPASS, OR (3) HIGHPASS FILTERING (1/2/3):"
01360 DO:ANS=INKEY$:UNTIL LEN(ANS)
01370 IF INSTR(1,"123",ANS)=0 THEN 1360 ELSE COLOR,15:PRINT@(74,14)ANS
01380 KF=VAL(ANS)
01390 LONG IF KF=1 OR KF=2
01400 COLOR,7:PRINT@(1,16)"H-F CUTOFF, ORDINATES/CYCLE:"
01410 COLOR,15:INPUT@(30,16);&6,"";ANS
01420 CH=VAL(ANS):IF CH<4 THEN 1410 ELSE F2=EXP(-(ATN(1)<<3)/CH)
01430 END IF
01440 LONG IF KF=1 OR KF=3
01450 COLOR,7:PRINT@(40,16)"L-F CUTOFF, ORDINATES/CYCLE:"
01460 COLOR,15:INPUT@(69,16);&6,"";ANS
01470 CL=VAL(ANS):IF CL<=1.5*CH THEN 1460 ELSE F1=EXP(-(ATN(1)<<3)/CL)
01480 END IF
01490 IF KF=1 THEN FAC=CL/(CL-CH)
01500 '
01510 '                 OPEN FILES AND BEGIN FILTERING
01520 '
01530 OPEN "I",1,BF(1):OPEN "O",2,BF(2)
01540 COLOR,7:ON ERROR GOSUB 65535
01550 LINE INPUT#1,A:PRINT@(30,18)A
01560 PRINT#2,A
01570 LINE INPUT#1,A:PRINT@(22,19)"MEASUREMENT UNITS ARE "+A
01580 PRINT#2,A
01590 PRINT@(34,21)"SAMPLES PROCESSED"
01600 PRINT@(23,23)"PRESS ANY KEY TO INTERRUPT PROGRAM"
01610 INPUT#1,X1(1):X2(1)=X1(1)
01620 '
01630 '************************************************************************
01640 '
01650 '                            MAIN LOOP
01660 '
01670 '************************************************************************
01680 '
01690 J=2:DO
01700 IF USR3(1) THEN GOSUB "CONCL"
01710 LINE INPUT#1,AX
01720 IF INSTR(1,AX,CHR$(4)) THEN "CONCL" ELSE X=VAL(AX)
01730 ON KF GOSUB "BP","LP","HP"
01740 PRINT#2,Y
01750 ON ERROR GOSUB "CONCL"
01760 PRINT@(28,21)UNS$(J)
01770 J=J+1:UNTIL INSTR(1,AX,CHR$(4))
01780 CLOSE 1:PRINT#2,CHR$(4)","CHR$(4)","CHR$(4):CLOSE 2
01790 LOCATE 1,23:CLS LINE:PRINT@(30,23)"CONVERSION FINISHED"
01800 END
01810 '
01820 '************************************************************************
01830 '
01840 '                         SUBROUTINES ETC.
01850 '
01860 '************************************************************************
01870 '
01880 '                            BANDPASS
01890 '
01900 "BP"
01910 X1(2)=(1-F1)*X+F1*X1(1):X2(2)=(1-F2)*X+F2*X2(1)
01920 Y=FAC*(X2(2)-X1(2))
01930 X1(1)=X1(2):X2(1)=X2(2)
01940 RETURN
01950 '
01960 '                            LOW-PASS
01970 '
01980 "LP"
01990 Y=(1-F2)*X+F2*X2(1)
02000 X2(1)=Y
02010 RETURN
02020 '
02030 '                            HIGH-PASS
02040 '
02050 "HP"
02060 X1(2)=(1-F1)*X+F1*X1(1)
02070 Y=X-X1(2)
02080 X1(1)=X1(2)
02090 RETURN
02100 '
02110 '                          END CONVERSION
02120 '
02130 "CONCL"
02140 IF ERROR=514 THEN LOCATE 1,25:CLS LINE:PRINT@(25,25)"PROCESSING TERMINATED; DISK FILLED"
02150 ERR0R=0:RETURN 1780
